/**************************************************************************
*
* Nom du projet     : Gauss
* Objet 				  : Rsolution d'un systme linaire  n quations
*							  n inconnues
*
***************************************************************************
*
* Rpertoire                : gauss\lib
* Nom du fichier            : detrang.c
* Nom du fichier d'analyse  : suivi\suivi.doc
* Auteur                    : David ROUSSE - Igor APARICI
* Date de creation          : 09 / 01 / 2000
* Date de mise a jour       :
* Valide par                : David ROUSSE
* Date de validation        : 11 / 01 / 2000
* Indice du module          : 0
*
***************************************************************************
*
***************************************************************************
*/

#include "perso.h"

/************************************************************
* NOM : detrang
* INDICE : 0
* AUTEUR : Igor APARICI - David Rousse
* DESCRIPTION : Dtermination du rang et du dterminant
*					 d'un systme linaire carr
*               par la mthode du pivot partiel c'est  dire
*					 recherche du premier pivot non nul et rduction sous le pivt.
*					 On ralise  chaque tape que des oprations de type addition
*					 sur les lignes du systme donc, au final, le dterminant du
*					 systme obtenu est le mme que celui de la matrice initiale
*					 (Ainsi, il devient inutile de compte le nombre de permutations
*					 de lignes).
*					 De plus, le rang du systme est gal au nombre de pivts non
*					 nuls trouvs.
*					 Les paramtres d'appel de cette fonction sont :
*						a : un tableau carr (la matrice du systme)
*						d : dterminant du systme (paramtre rsultat)
*						n : l'ordre de la matrice
*
*					 Elle renvoie la valeur du rang et, quand la valeur renvoye est
*					 gale  la taille du systme, la valeur du dterminant
*					 du systme se trouve dans determinant
*
*
* SYNTAXE : int detrang(double **A, double *B, int *determinant, int taille);
************************************************************
*/
int detrang(double **A, int *determinant, int taille)
{

	int i, j, col=0, pivotnultrouve=true, pivotnul, ligne, rang=0;
 	double coeffareduire, **systeme; /* systeme contient tout */
   													 /* le systme linaire   */


 	/* allocation memoire */
 	systeme = allouer(taille,taille+1);


 	/* copie du premier membre dans systeme */
 	for(i=0;i<taille;i++)
 	{
 		for(j=0;j<taille;j++)
   	  	systeme[i][j]=A[i][j];
 	}

   /* debut du traitement */

 	while(pivotnultrouve && col<taille) /* Traitement de la col-ime colonne */
 	{

      /* recherche du premier pivot non nul parmi */
      /* systeme[col][col]...systeme[taille][col] */
      pivotnul=true;
  		ligne=col;

  		while (ligne<taille && pivotnul)
      {
         if(systeme[ligne][col]==0)
         	ligne++;
         else
         	pivotnul=false;
      } /* fin de la recherche du pivot */

      if(pivotnul) /* systeme[ligne][col]=0, pour ligne=col,...,taille */
        	pivotnultrouve=false;  /* Le systme n'a pas de solution */
      else   /* systeme[ligne][col] contient le premier pivot non nul */
      {
         /* un pivot non nul a t trouv donc on incremente le rang */
         rang++;

         /* on ramene le pivot sur la diagonale */
      	if(ligne>col) /* On amne le pivot dans systeme[col][col] */
         {
           	for(j=col;j<taille;j++)
               systeme[col][j]+=systeme[ligne][j];

      		/* Le pivot est sur la diagonale systeme[col][col]              */
         	/* et dans systeme[ligne][col]											 */
      		/* Les coefficients systeme[col+1][col],...,systeme[ligne][col] */
      		/* sont dj nuls                                               */
      		/* On rduit les lignes de ligne  taille-1                   */
      		for (i=ligne;i<taille;i++)
      		{
            	/* pour reduire, on realise l'operation
					   ligne i<-ligne i - systeme[i][col]/systeme[col][col]*ligne col
                                                                              */
         		coeffareduire=systeme[i][col]/systeme[col][col];
         		for(j=col;j<taille;j++)
         			systeme[i][j]-=(coeffareduire*systeme[col][j]);

      		}
         }
         else /* ligne=col donc le pivt est dj sur la doagonale */
         {    /* on limine sous le pivot en partant de col+1      */
           	for(i=col+1;i<taille;i++)
            {
               /* on ralise ligne i<-ligne i -                               */
               /*                 systeme[i][col]/systeme[col][col]*ligne col */
            	coeffareduire = systeme[i][col]/systeme[col][col];
               for(j=col;j<taille;j++)
               	systeme[i][j]-=(coeffareduire*systeme[col][j]);
         	}
         }

      	/* On passe  la colonne suivante */
      	col++;
 		}
	} /* On continue tant que le systme n'est pas singulier */

	/* Calcul du vecteur solution du systme */
	if(pivotnultrouve)
	{
      /* systeme contient une matrice triangulaire suprieure de mme */
      /* dterminant que le systme initial.									 */
      /* Ce dterminant est le produit des pivts c'est  dire des    */
      /* coefficients diagonaux de systeme 									 */
      *determinant = 1;
      for (i=0;i<taille;i++)
      	(*determinant)*=systeme[i][i]; /* on forme le produit des termes */
         										 /* diagonaux                      */
	}
   else
   	*determinant = 0; /* le sytme est singulier et son dterminant est nul */

	/* liberation memoire */
	liberermat(systeme,taille);

	return rang; /* retourne la valeur du rang du systme */
}

