/**************************************************************************
*
* Nom du projet     : Gauss
* Objet 				  : Rsolution d'un systme linaire  n quations
*							  n inconnues
*
***************************************************************************
*
* Rpertoire                : gauss\lib
* Nom du fichier            : pivotgauss.c
* Nom du fichier d'analyse  : suivi\suivi.doc
* Auteur                    : David ROUSSE - Igor APARICI
* Date de creation          : 04 / 01 / 2000
* Date de mise a jour       :
* Valide par                : 11 / 01 / 2000
* Date de validation        : Igor APARICI
* Indice du module          : 0
*
***************************************************************************
*
***************************************************************************
*/

#include "perso.h"

/************************************************************
* NOM : pivotgauss
* INDICE : 0
* AUTEUR : Igor APARICI - David Rousse
* DESCRIPTION : Rsolution d'un systme linaire carr
*               par la mthode du pivot de Jordan-Gauss c'est  dire
*					 recherche du premier pivot non nul, normalisation de la
*					 ligne du pivt et rduction sous le pivt.
*					 Les paramtres d'appel de cette fonction sont :
*						a : un tableau carr (la matrice du systme)
*						b : un tableau unidimensionnel (le second membre du systme)
*						solution : un tableau unidimensionnel pour le vecteur solution
*						n : l'ordre de la matrice
*
*					 Elle renvoie la valeur 1 si la matrice est numriquement
*					 inversible et la valeur 0 sinon; quand la valeur renvoye est 1,
*					 la solution du systme se trouve dans le tableau solution.
*
*
* SYNTAXE : int pivotgauss(double **A, double *B, double *solution, int taille);
************************************************************
*/
int pivotgauss(double **A,double *B, double *solution, int taille)
{

	int i, j, col=0, pivotnultrouve=true, pivotnul, ligne;
 	double pivot, coeffareduire, **systeme; /* systeme contient tout */
   													 /* le systme linaire   */


 	/* allocation memoire */
 	systeme = allouer(taille,taille+1);


 	/* copie des premier et second membres dans systeme */
 	for(i=0;i<taille;i++)
 	{
 		for(j=0;j<taille;j++)
   	  	systeme[i][j]=A[i][j];

  		systeme[i][taille]=B[i];
 	}

   /* debut du traitement */

 	while(pivotnultrouve && col<taille) /* Traitement de la col-ime colonne */
 	{

      /* recherche du premier pivot non nul parmi */
      /* systeme[col][col]...systeme[taille][col] */
      pivotnul=true;
  		ligne=col;

  		while (ligne<taille && pivotnul)
      {
         if(systeme[ligne][col]==0)
         	ligne++;
         else
         	pivotnul=false;
      } /* fin de la recherche du pivot */

      if(pivotnul) /* systeme[ligne][col]=0, pour ligne=col,...,taille */
        	pivotnultrouve=false;  /* Le systme n'a pas de solution */
      else   /* systeme[ligne][col] contient le premier pivot non nul */
      {
         /* on ramene le pivot sur la diagonale */
      	if(ligne>col) /* On change les lignes ligne et col */
         {
           	for(j=col;j<taille;j++)
            {
             	/* On change systeme[ligne][j] et systeme[col][j] */
               pivot=systeme[ligne][j];
               systeme[ligne][j]=systeme[col][j];
            	systeme[col][j]=pivot;
            }

            /* On change systeme[ligne][taille] et systeme[col][taille+1] */
            pivot=systeme[ligne][taille];
            systeme[ligne][taille]=systeme[col][taille];
            systeme[col][taille]=pivot;
         }

      	/* Le pivot est sur la diagonale systeme[col][col]                 */
      	/* et les coefficients systeme[col+1][col],...,systeme[ligne][col] */
      	/* sont dj nuls                                                  */

         pivot=systeme[col][col];
         /* On normalise la ligne col de col  taille                       */
         for(j=col;j<taille+1;j++)
         	systeme[col][j]=systeme[col][j]/pivot;

      	/* On rduit les lignes de ligne+1  taille-1                      */
      	pivot=1;
      	for (i=ligne+1;i<taille;i++)
      	{
            /* pour reduire, on realise l'operation
				/* ligne i<-ligne i - systeme[i][col]/systeme[col][col]*ligne col */
            /* ce qui revient  faire
      		/* ligne i<-systeme[col][col]*ligne i - systeme[i][col]*ligne col */
            /* et on vite ainsi de diviser par un pivot proche de 0          */

      		/* Premier membre                                                 */
         	coeffareduire=systeme[i][col];
         	for(j=col;j<taille;j++)
         		systeme[i][j]=pivot*systeme[i][j]-coeffareduire*systeme[col][j];

         	/* Second membre */
         	systeme[i][taille]=pivot*systeme[i][taille]-
         												coeffareduire*systeme[col][taille];
      	}

      	/* On passe  la colonne suivante */
      	col++;
 		}
	} /* On continue tant que le systme n'est pas singulier */

	/* Calcul du vecteur solution du systme */
	if(pivotnultrouve)
	{
		for (i=taille-1;i>=0;i--)
     	{
   		pivot=0;
      	for(j=i+1;j<taille;j++)
      		pivot+=systeme[i][j]*solution[j];
         /* Si i=n, on sort de suite avec pivot=0 */
      	solution[i]=systeme[i][taille]-pivot;
      }
	}

	/* liberation memoire */
	liberermat(systeme,taille);

	return pivotnultrouve; /* retourne 1 si la solution existe, 0 sinon */
}

